-- GridStatusRole.lua
--
-- Created By : Greltok

--{{{ Libraries
local L = AceLibrary("AceLocale-2.2"):new("GridStatusRole")
local talentLib = LibStub:GetLibrary("LibGroupTalents-1.0")
local GridRoster = Grid:GetModule("GridRoster")
--}}}

local GridStatusRole = Grid:GetModule("GridStatus"):NewModule("Role")
GridStatusRole.menuName = L["Role"]

local rolestatus = {
    caster = {
		text = L["Caster"],
		icon = "Interface\\Icons\\Spell_Fire_BlueFlameBolt",
	},
	healer = {
		text = L["Healer"],
		icon = "Interface\\Icons\\Spell_Holy_Renew",
	},
    melee = {
		text = L["Melee"],
		icon = "Interface\\Icons\\Ability_DualWield",
	},
    tank = {
		text = L["Tank"],
		icon = "Interface\\Icons\\Ability_Warrior_ShieldMastery",
	},
}

--{{{ AceDB defaults
GridStatusRole.defaultDB = {
    debug = false,
    role = {
        text = L["Role"],
        enable = true,
        color = { r = 1, g = 1, b = 1, a = 1 },
        priority = 10,
        range = false,
        hideInCombat = false,
        colors = {
            melee = { r = 1, g = 1, b = 0, a = 1 },
            caster = { r = 0, g = 0, b = 1, a = 1 },
            healer = { r = 0, g = 1, b = 0, a = 1 },
            tank = { r = 1, g = 0, b = 0, a = 1 },
        },
        filter = {
            melee = true,
            caster = true,
            healer = true,
            tank = true,
        },
    },
}
--}}}

GridStatusRole.options = false

local function getrolecolor(role)
    local color = GridStatusRole.db.profile.role.colors[role]
    return color.r, color.g, color.b, color.a
end

local function setrolecolor(role, r, g, b, a)
    local color = GridStatusRole.db.profile.role.colors[role]
    color.r = r
    color.g = g
    color.b = b
    color.a = a or 1
    GridStatus:TriggerEvent("Grid_ColorsChanged")
end

local function getrolefilter(role)
    return GridStatusRole.db.profile.role.filter[role] ~= false
end

local function setrolefilter(role, v)
    GridStatusRole.db.profile.role.filter[role] = v
    GridStatusRole:RoleCheckAll()
end

--{{{ additional options
local roleOptions = {
    ["caster"] = {
        type = "color",
        name = L["Caster color"],
        desc = L["Color for Casters."],
        order = 86,
        hasAlpha = true,
        get = function () return getrolecolor("caster") end,
        set = function (r, g, b, a) setrolecolor("caster", r, g, b, a) end,
    },
    ["healer"] = {
        type = "color",
        name = L["Healer color"],
        desc = L["Color for Healers."],
        order = 87,
        hasAlpha = true,
        get = function () return getrolecolor("healer") end,
        set = function (r, g, b, a) setrolecolor("healer", r, g, b, a) end,
    },
    ["melee"] = {
        type = "color",
        name = L["Melee color"],
        desc = L["Color for Melee."],
        order = 88,
        hasAlpha = true,
        get = function () return getrolecolor("melee") end,
        set = function (r, g, b, a) setrolecolor("melee", r, g, b, a) end,
    },
    ["tank"] = {
        type = "color",
        name = L["Tank color"],
        desc = L["Color for Tanks."],
        order = 89,
        hasAlpha = true,
        get = function () return getrolecolor("tank") end,
        set = function (r, g, b, a) setrolecolor("tank", r, g, b, a) end,
    },
    ["filter"] = {
        type = "group",
        name = L["Role filter"],
        desc = L["Show status for the selected roles."],
        order = 90,
        args = {
            ["caster"] = {
                type = "toggle",
                name = L["Caster"],
                desc = L["Show on Caster."],
                get = function () return getrolefilter("caster") end,
                set = function (v) setrolefilter("caster", v) end,
            },
            ["healer"] = {
                type = "toggle",
                name = L["Healer"],
                desc = L["Show on Healer."],
                get = function () return getrolefilter("healer") end,
                set = function (v) setrolefilter("healer", v) end,
            },
            ["melee"] = {
                type = "toggle",
                name = L["Melee"],
                desc = L["Show on Melee."],
                get = function () return getrolefilter("melee") end,
                set = function (v) setrolefilter("melee", v) end,
            },
            ["tank"] = {
                type = "toggle",
                name = L["Tank"],
                desc = L["Show on Tank."],
                get = function () return getrolefilter("tank") end,
                set = function (v) setrolefilter("tank", v) end,
            },
        },
    },
    ["hideInCombat"] = {
        type = "toggle",
        name = L["Hide in combat"],
        desc = L["Hide roles while in combat."],
        order = 91,
        get = function() return GridStatusRole.db.profile.role.hideInCombat end,
        set = function()
            local settings = GridStatusRole.db.profile.role
            settings.hideInCombat = not settings.hideInCombat
            if settings.enable then
                if settings.hideInCombat then
                    GridStatusRole:RegisterEvent("Grid_EnteringCombat")
                    GridStatusRole:RegisterEvent("Grid_LeavingCombat")
                else
                    GridStatusRole:UnregisterEvent("Grid_EnteringCombat")
                    GridStatusRole:UnregisterEvent("Grid_LeavingCombat")
                end
                GridStatusRole:RoleCheckAll()
            end
        end,
    },

    ["color"] = false,
}
--}}}

function GridStatusRole:OnInitialize()
    self.super.OnInitialize(self)
    self:RegisterStatus("role", L["Role"], roleOptions, true)
end

function GridStatusRole:OnStatusEnable(status)
	if status == "role" then
		talentLib.RegisterCallback(self, "LibGroupTalents_RoleChange")
		talentLib.RegisterCallback(self, "LibGroupTalents_Add")
		if self.db.profile.role.hideInCombat then
            self:RegisterEvent("Grid_EnteringCombat")
            self:RegisterEvent("Grid_LeavingCombat")
        end
		self:RoleCheckAll()
	end
end

function GridStatusRole:OnStatusDisable(status)
	if status == "role" then
		talentLib.UnregisterCallback(self, "LibGroupTalents_RoleChange")
		talentLib.UnregisterCallback(self, "LibGroupTalents_Add")
		if self.db.profile.role.hideInCombat then
            self:UnregisterEvent("Grid_EnteringCombat")
            self:UnregisterEvent("Grid_LeavingCombat")
        end
		self.core:SendStatusLostAllUnits("role")
	end
end

function GridStatusRole:Reset()
    self.super.Reset(self)
    self:RoleCheckAll()
    if self.db.profile.role.hideInCombat then
        GridStatusRole:RegisterEvent("Grid_EnteringCombat")
        GridStatusRole:RegisterEvent("Grid_LeavingCombat")
    else
        GridStatusRole:UnregisterEvent("Grid_EnteringCombat")
        GridStatusRole:UnregisterEvent("Grid_LeavingCombat")
    end
end

function GridStatusRole:Grid_EnteringCombat()
    local settings = self.db.profile.role
    if settings.enable and settings.hideInCombat then
        self.core:SendStatusLostAllUnits("role")
    end
end

function GridStatusRole:Grid_LeavingCombat()
    local settings = self.db.profile.role
    if settings.enable and settings.hideInCombat then
        self:RoleCheckAll()
    end
end

function GridStatusRole:RoleCheckAll()
    local settings = self.db.profile.role
    if settings.enable and ( not settings.hideInCombat or not Grid.inCombat ) then
        for guid in GridRoster:IterateRoster() do
            self:RoleCheck(guid)
        end
	else
	   self.core:SendStatusLostAllUnits("role")
    end
end

function GridStatusRole:LibGroupTalents_RoleChange(event, guid, unit, role)
    self:RoleCheck(guid, role)
end

function GridStatusRole:LibGroupTalents_Add(event, guid)
    self:RoleCheck(guid)
end

function GridStatusRole:RoleCheck(guid, role)
    local gained
    local settings = self.db.profile.role
    if settings.enable and ( not settings.hideInCombat or not Grid.inCombat ) then
        role = role or talentLib:GetGUIDRole(guid)
        if role and settings.filter[role] then
            local status = rolestatus[role]
            self.core:SendStatusGained(guid,
                                        "role",
                                        settings.priority,
                                        (settings.range and 40),
                                        settings.colors[role],
                                        status.text,
                                        nil,
                                        nil,
                                        status.icon)
            gained = true
        end
	end
	if not gained then
		self.core:SendStatusLost(guid, "role")
	end
end
